/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 2010-2011 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

Application
    LTSThermoParcelFoam

Description
    This solver is derived from OpenFOAM's LTSReactingParcelFoam.
    Local time stepping (LTS) solver for steady, compressible, laminar or
    turbulent non-reacting flow with multiphase Lagrangian parcels.

    The focus of this solver is to model the strong coupled thermal interaction
    of evaporating water droplets in a quenching device for cooling of hot gas.

    Further more the use of the local time stepping convergence acceleration
    method is an essential feature.


    There are several changes with regard to the original LTSReactingParcelFoam:

    The solver uses psiThermo classes instead of rhoThermo classes. This makes
    it easy to combine LTSThermoParcelFoam with the original reactingParcelFoam.
    I.e. you can use LTSThermoParcelFoam to initialize your simulation and
    reactingParcelFoam as a successor which provides wonderful particle pathways.

    The chemical reactions are removed from the solver. However the data
    structures and classes from the psiChemistryModel.H are still in use to do
    the multi species management.

    Porous zones, radiation and explicit source terms are removed, too. However
    if you need these features it's easy to add them again.


    Parameters for LTS are read from tables in system/controlDict, for example:
    maxCourantTable table
    (
        (0      0.2)
        (500    0.5)
        (20000  0.5)
    )
    There's a linear interpolation for these values. The idea is that you can
    start your simulation with a stable configuration, but speed up the
    convergence after a couple of iterations.
    Same procedure for: rDeltaTSmoothingCoeff, maxDeltaT and alphaTemp

Contact
    martin_becker@dhcae-tools.de


\*---------------------------------------------------------------------------*/

#include "fvCFD.H"
#include "hReactionThermo.H"
#include "turbulenceModel.H"
#include "basicReactingMultiphaseCloud.H"
#include "RASModel.H"
#include "wallDist.H"
#include "compressible/RAS/RASModel/RASModel.H"
#include "mutkWallFunction/mutkWallFunctionFvPatchScalarField.H"
#include "psiChemistryCombustionModel.H"
//#include "psiChemistryModel.H"
#include "SLGThermo.H"
#include "fvcSmooth.H"
#include "pimpleControl.H"
#include "DataEntry.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

int main(int argc, char *argv[])
{
    argList::validOptions.insert("logFrequency", "25");

    #include "setRootCase.H"
    #include "createTime.H"
    #include "createMesh.H"
    #include "readGravitationalAcceleration.H"

    pimpleControl pimple(mesh);

    #include "writeLogInit.H"
    #include "readTimeControls.H"
    #include "readAdditionalSolutionControls.H"
    #include "createFields.H"
    #include "createClouds.H"
    #include "initContinuityErrs.H"

    // * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

    Info<< "\nStarting time loop\n" << endl;

    while (runTime.run())
    {
        #include "readAdditionalSolutionControls.H"
        #include "readTimeControls.H"

        runTime++;

        Info<< "Time = " << runTime.timeName() << nl << endl;

        timeBefore = runTime.elapsedCpuTime();
        parcels.evolve();
        timeAfter = runTime.elapsedCpuTime();
        timeStatsHashTable["CPUTimeCloud"] += timeAfter - timeBefore;
        timeStatsHashTable["countCloud"] += 1;

        #include "timeScales.H"

        timeBeforeContinuousPhase = runTime.elapsedCpuTime();
        #include "rhoEqn.H"

        // --- Pressure-velocity PIMPLE corrector loop
         while (pimple.loop())
        {
            turbulence->correct();

            #include "UEqn.H"
            #include "YEqn.H"
            #include "hsEqn.H"

            // --- PISO loop
            while (pimple.correct())
            {
                #include "pEqn.H"
            }
        }
        timeAfterContinuousPhase = runTime.elapsedCpuTime();
        timeStatsHashTable["CPUTimeContinuousPhase"] += timeAfterContinuousPhase - timeBeforeContinuousPhase;
        timeStatsHashTable["countContinuousPhase"] += 1;

        if (runTime.write())
        {
            //#include "write.H"
        }
        #include "writeTimeStats.H"

        Info<< "ExecutionTime = " << runTime.elapsedCpuTime() << " s"
            << "  ClockTime = " << runTime.elapsedClockTime() << " s"
            << nl << endl;
    }
    Info<< "End\n" << endl;

    return(0);
}


// ************************************************************************* //
